@@ -7,18 +7,16 @@ use rowan::SyntaxElement;
 
 use crate::{
     algo::neighbor,
-    ast::{
-        self,
-        edit::{AstNodeEdit, IndentLevel},
-        make, GenericParamsOwner,
-    },
+    ast::{self, edit::IndentLevel, make, GenericParamsOwner},
     ted::{self, Position},
-    AstNode, AstToken, Direction, SyntaxNode,
+    AstNode, AstToken, Direction,
+    SyntaxKind::{ATTR, COMMENT, WHITESPACE},
+    SyntaxNode,
 };
 
 use super::NameOwner;
 
-pub trait GenericParamsOwnerEdit: ast::GenericParamsOwner + AstNodeEdit {
+pub trait GenericParamsOwnerEdit: ast::GenericParamsOwner {
     fn get_or_create_generic_param_list(&self) -> ast::GenericParamList;
     fn get_or_create_where_clause(&self) -> ast::WhereClause;
 }
@@ -196,6 +194,32 @@ fn create_generic_param_list(position: Position) -> ast::GenericParamList {
     gpl
 }
 
+pub trait AttrsOwnerEdit: ast::AttrsOwner {
+    fn remove_attrs_and_docs(&self) {
+        remove_attrs_and_docs(self.syntax());
+
+        fn remove_attrs_and_docs(node: &SyntaxNode) {
+            let mut remove_next_ws = false;
+            for child in node.children_with_tokens() {
+                match child.kind() {
+                    ATTR | COMMENT => {
+                        remove_next_ws = true;
+                        child.detach();
+                        continue;
+                    }
+                    WHITESPACE if remove_next_ws => {
+                        child.detach();
+                    }
+                    _ => (),
+                }
+                remove_next_ws = false;
+            }
+        }
+    }
+}
+
+impl<T: ast::AttrsOwner> AttrsOwnerEdit for T {}
+
 impl ast::GenericParamList {
     pub fn add_generic_param(&self, generic_param: ast::GenericParam) {
         match self.generic_params().last() {
@@ -359,14 +383,10 @@ impl ast::MatchArmList {
         let mut elements = Vec::new();
         let position = match self.arms().last() {
             Some(last_arm) => {
-                let comma = last_arm
-                    .syntax()
-                    .siblings_with_tokens(Direction::Next)
-                    .find(|it| it.kind() == T![,]);
-                if needs_comma(&last_arm) && comma.is_none() {
-                    elements.push(make::token(SyntaxKind::COMMA).into());
+                if needs_comma(&last_arm) {
+                    ted::append_child(last_arm.syntax(), make::token(SyntaxKind::COMMA));
                 }
-                Position::after(comma.unwrap_or_else(|| last_arm.syntax().clone().into()))
+                Position::after(last_arm.syntax().clone())
             }
             None => match self.l_curly_token() {
                 Some(it) => Position::after(it),
@@ -377,12 +397,12 @@ impl ast::MatchArmList {
         elements.push(make::tokens::whitespace(&format!("\n{}", indent)).into());
         elements.push(arm.syntax().clone().into());
         if needs_comma(&arm) {
-            elements.push(make::token(SyntaxKind::COMMA).into());
+            ted::append_child(arm.syntax(), make::token(SyntaxKind::COMMA));
         }
         ted::insert_all(position, elements);
 
         fn needs_comma(arm: &ast::MatchArm) -> bool {
-            arm.expr().map_or(false, |e| !e.is_block_like())
+            arm.expr().map_or(false, |e| !e.is_block_like()) && arm.comma_token().is_none()
         }
     }
 }
@@ -463,6 +483,24 @@ fn normalize_ws_between_braces(node: &SyntaxNode) -> Option<()> {
     Some(())
 }
 
+pub trait Indent: AstNode + Clone + Sized {
+    fn indent_level(&self) -> IndentLevel {
+        IndentLevel::from_node(self.syntax())
+    }
+    fn indent(&self, level: IndentLevel) {
+        level.increase_indent(self.syntax());
+    }
+    fn dedent(&self, level: IndentLevel) {
+        level.decrease_indent(self.syntax());
+    }
+    fn reset_indent(&self) {
+        let level = IndentLevel::from_node(self.syntax());
+        self.dedent(level);
+    }
+}
+
+impl<N: AstNode + Clone> Indent for N {}
+
 #[cfg(test)]
 mod tests {
     use std::fmt;
@@ -502,4 +540,22 @@ mod tests {
         check_create_gpl::<ast::Enum>("enum E", "enum E<>");
         check_create_gpl::<ast::Enum>("enum E {", "enum E<> {");
     }
+
+    #[test]
+    fn test_increase_indent() {
+        let arm_list = ast_mut_from_text::<ast::Fn>(
+            "fn foo() {
+    ;
+    ;
+}",
+        );
+        arm_list.indent(IndentLevel(2));
+        assert_eq!(
+            arm_list.to_string(),
+            "fn foo() {
+            ;
+            ;
+        }",
+        );
+    }
 }
